using System;
using System.Drawing;
using System.Threading;

namespace Photo_Editor_Application
{
    /// <summary>
    /// Abstract Component class.
    /// Every graphic primitive and picture has to derive from 
    /// this class. It provides the common interfaces the 
    /// application can use.
    /// </summary>
    public abstract class GraphicsComponent
    {
        /// <summary>
        /// Constructor, initializes the start and end point 
        /// of each primitive to be the origin.
        /// </summary>
        public GraphicsComponent()
        {
            startPoint.X = startPoint.Y = 0;
            endPoint.X = endPoint.Y = 0;
        }

        /// <summary>
        /// Abstract method to be implemented by the derived
        /// classes to draw the corresponding primitive into
        /// the drawing surface.
        /// </summary>
        public abstract void Draw();

        /// <summary>
        /// Method to add a component to the list of components.
        /// Default behavior is to do nothing.
        /// </summary>
        public virtual void Add(GraphicsComponent comp){}

        /// <summary>
        /// Method to remove a component from the list of components.
        /// Default behavior is to do nothing.
        /// </summary>
        public virtual void Remove(GraphicsComponent comp){}

        /// <summary>
        /// Accessor method for the static deviceContext
        /// field that holds the graphics reference to
        /// the drawing surface.
        /// </summary>
        public static Graphics DeviceContext
        {
            get
            {
                return deviceContext;
            }
            set
            {
                deviceContext = value;
            }
        }

        /// <summary>
        /// Holds the drawing surface.
        /// </summary>
        private static Graphics deviceContext;

        /// <summary>
        /// Accessor method for the coordinates of
        /// the upper left corner of the primitive, or 
        /// the start point of a line.
        /// </summary>
        public Point StartPoint
        {
            get
            {
                return startPoint;
            }
            set
            {
                startPoint = value;
            }
        }

        /// <summary>
        /// Accessor method for the coordinates of
        /// the lower right hand corner of the primitive, or 
        /// the end point of a line.
        /// </summary>
        public Point EndPoint
        {
            get
            {
                return endPoint;
            }
            set
            {
                endPoint = value;
            }
        }
    
        /// <summary>
        /// Calculates the coordinates within the display 
        /// window from a 
        /// point relative to the image.
        /// </summary>
        /// <param name="imagePoint">Point in image 
        /// coordinates</param>
        public Point CalcViewportCoords(Point imagePoint)
        {
            Point viewportPoint = new Point(imagePoint.X - viewportCoords.X, imagePoint.Y - viewportCoords.Y);
            return viewportPoint;
        }

        /// <summary>
        /// Set method for the coordinates of
        /// the image within the display window.
        /// </summary>
        public static Point ViewportCoords
        {
            set
            {
                viewportCoords = value;
            }
        }
        #region Do It Yourself Chapter 6
        /// <summary>
        /// Checks if component is picked.
        /// Default implementation checks if
        /// start or end point is picked.
        /// </summary>
        public virtual bool IsPicked(int x, int y)
        {
            const int delta = 5;
            if(((Math.Abs(StartPoint.X - x) < delta) && 
                (Math.Abs(StartPoint.Y - y) < delta)) || 
                ((Math.Abs(EndPoint.X - x) < delta) && 
                (Math.Abs(EndPoint.Y - y) < delta)))
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        #endregion
        private Point endPoint;
        private Point startPoint;
        private static Point viewportCoords;
    }
}
